public class Test {
  public static void main(String[] args) throws Exception {
    String[] cmdarray = new String[args.length];
    System.arraycopy(args, 1, cmdarray, 0, args.length - 1);
    java.io.File testDir = new java.io.File(args[0]).getAbsoluteFile();
    java.util.Scanner tests = new java.util.Scanner(new java.io.File(testDir, "tests.txt"));
    long total = 0;
    long max = Long.MIN_VALUE;
    long min = Long.MAX_VALUE;
    boolean anyNotPassed = false;
    boolean anyCarriageReturns = false;
    boolean anyErrorBytes = false;
    while (tests.hasNext()) {
      java.io.File input = new java.io.File(testDir, tests.next());
      String correct_md5 = tests.next();
      StringBuilder status = new StringBuilder();
      System.out.print(input.getName());
      System.out.print(":\t");
      if (input.canRead()) {
        cmdarray[cmdarray.length - 1] = input.getPath();
        TestResult result = exec(cmdarray);
        total += result.time;
        max = Math.max(max, result.time);
        min = Math.min(min, result.time);
        if (result.md5.equals(correct_md5) && !result.errorBytes) {
          System.out.print("PASS");
        } else {
          System.out.print("FAIL");
          anyNotPassed = true;
        }
        System.out.print(" (");
        System.out.print(result.time);
        System.out.print(" ms)");
        if (result.carriageReturn) {
          System.out.print(", carriage return");
          anyCarriageReturns = true;
        }
        if (result.errorBytes) {
          System.out.print(", error bytes");
          anyErrorBytes = true;
        }
      } else {
        System.out.print("MISSING");
        anyNotPassed = true;
      }
      System.out.println();
    }
    tests.close();
    System.out.println();
    System.out.println("Summary");
    if (anyNotPassed) {
      System.out.println("One or more tests not passed");
    } else {
      System.out.println("All tests passed");
    }
    System.out.print("total: ");
    System.out.print(total);
    System.out.println(" ms");
    System.out.print("max: ");
    System.out.print(max);
    System.out.println(" ms");
    System.out.print("min: ");
    System.out.print(min);
    System.out.println(" ms");
    if (anyCarriageReturns || anyErrorBytes) {
      System.out.println();
      if (anyCarriageReturns) {
        System.out.println("carriage return - use Unix-style line separators.");
      }
      if (anyErrorBytes) {
        System.out.println("error bytes - do not print to the standard error stream.");
      }
    }
    if (anyNotPassed) {
      System.exit(1);
    }
  }
  static TestResult exec(String[] cmdarray) throws Exception {
    long start = System.currentTimeMillis();
    Process p = Runtime.getRuntime().exec(cmdarray);
    p.getOutputStream().close();
    Gobbler g = new Gobbler();
    g.err = p.getErrorStream();
    Thread t = new Thread(g);
    t.start();
    int len;
    java.io.InputStream in = p.getInputStream();
    byte[] buf = new byte[16384];
    TestResult result = new TestResult();
    java.security.MessageDigest md = java.security.MessageDigest.getInstance("MD5");
    while ((len = in.read(buf)) != -1) {
      for (int i = 0; i < len; i++) {
        if (buf[i] == '\r') {
          result.carriageReturn = true;
        }
      }
      md.update(buf, 0, len);
    }
    t.join();
    result.errorBytes = g.errorBytes;
    p.waitFor();
    result.time = System.currentTimeMillis() - start;
    StringBuilder md5 = new StringBuilder();
    for (byte b : md.digest()) {
      md5.append(String.format("%02x", b & 0xff));
    }
    result.md5 = md5.toString();
    return result;
  }
}
class TestResult {
  String md5;
  long time;
  boolean carriageReturn = false;
  boolean errorBytes;
};
class Gobbler implements Runnable {
  public void run() {
    byte[] buf = new byte[16384];
    boolean eb = false;
    try {
      while (err.read(buf) > 0) {
        eb = true;
      }
    } catch (Exception e) {
      throw new RuntimeException(e);
    }
    errorBytes = eb;
  }
  java.io.InputStream err;
  boolean errorBytes;
}
